# Generated by Django 2.1.5 on 2019-08-11 23:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oauth2', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='client',
            name='auto_authorize',
            field=models.BooleanField(blank=True, default=False),
        ),
        migrations.AlterField(
            model_name='client',
            name='client_type',
            field=models.IntegerField(choices=[(0, 'Confidential (Web applications)'), (1, 'Public (Native and JS applications)')]),
        ),
        migrations.AlterField(
            model_name='client',
            name='redirect_uri',
            field=models.URLField(help_text="Your application's callback URL"),
        ),
        migrations.AlterField(
            model_name='client',
            name='url',
            field=models.URLField(help_text="Your application's URL."),
        ),
        migrations.AlterField(
            model_name='scope',
            name='description',
            field=models.CharField(blank=True, default='', max_length=256),
        ),
        migrations.AlterField(
            model_name='scope',
            name='name',
            field=models.CharField(max_length=50, primary_key=True, serialize=False),
        ),
    ]
